#include <stm32f0xx.h>
#include "i2c_soft.h"
#include <stdlib.h>
#include <string.h>
#include "font16v.h"

#define LCD       (*(__IO uint8_t *)((uint32_t)&(GPIOA->ODR) + 1))
#define LCD_DC_H	GPIOB->BSRR |= GPIO_BSRR_BS_3
#define LCD_DC_L	GPIOB->BSRR |= GPIO_BSRR_BR_3
#define LCD_WR_H	GPIOB->BSRR |= GPIO_BSRR_BS_1
#define LCD_WR_L	GPIOB->BSRR |= GPIO_BSRR_BR_1
#define CHAR_H    14 
#define CHAR_W    8
#define sky     	0x54fb
#define black   	0x0000
#define white   	0xFFFF
#define green   	0x07E0
#define blue    	0x001F
#define red     	0xF800
#define yellow  	0xDEE0
#define orange  	0xAB00
#define pink    	0xF97F
#define brown   	0x8200
#define grey    	0x8410
#define violet 	 	0x91D4
#define transp  	0x0001
//Depend PCB
#define Udd				2980	// Udd mV    2.98V
#define Rsht			110		// 90>>13 = 0.011 Ohm				110->.013
#define Ku				187		// 187>>8 = 0.7305
#define Ki				198		// 198>>8 = 0.7734
#define I0				201		// ADC data for In = 0
#define KIset			241		// 237>>8 = 0.9258		237
#define I0set			146		// Uopa mV for In = 0		146
#define KUset			1			// 1>>7 = 0.001953
#define U0set			2			// Ref U0 for PWM
//Mode initialization
#define TRIG			1
#define CONST			0
#define USET			0
#define ISET			1
#define SSET			2
#define GSET			3

uint8_t dd_sec = 0,Dsec_fl = 0,Key,Key_pow = 1,Guard = 0,Gmode = 0,OutEN = 0,ADC_ptr = 0,set_mode = USET,stln,p;
volatile uint16_t adc_res[2],adc_data[2][32],ADC_buff[2];
uint16_t Ucol = sky,Icol = yellow,Scol = green,Gcol = red;
volatile signed int In,Un,adc_sum[2] = {0,0},S;
signed int Iset = 2000,Uset = 500,temp;
char txt[10];
 
void init_gpio(void)
{
		RCC->AHBENR|=RCC_AHBENR_GPIOAEN;
		RCC->AHBENR|=RCC_AHBENR_GPIOBEN;
		
		GPIOA->MODER |=  GPIO_MODER_MODER0;			//Vout
		GPIOA->MODER |=  GPIO_MODER_MODER1;			//Iout
		GPIOA->MODER &=~  GPIO_MODER_MODER2;		//sw
		
		GPIOA->MODER &=~  GPIO_MODER_MODER3;		//Key1
		GPIOA->MODER &=~  GPIO_MODER_MODER4;		//Key2
		GPIOA->MODER &=~  GPIO_MODER_MODER5;		//Key3
		
		GPIOA->MODER &=~  GPIO_MODER_MODER6;		
		GPIOA->MODER |= GPIO_MODER_MODER6_1;		// buzz
		GPIOA->OTYPER &=~ GPIO_OTYPER_OT_6;			//PUSHPULL
		GPIOA->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR6;  
		GPIOA->AFR[0] |= 0x05000000;
		
		GPIOA->MODER &=~  GPIO_MODER_MODER7;		
		GPIOA->MODER |= GPIO_MODER_MODER7_1;		// Uset
		GPIOA->OTYPER &=~ GPIO_OTYPER_OT_7;			//PUSHPULL
		GPIOA->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR7;  
		GPIOA->AFR[0] |= 0x10000000;
		
		GPIOA->MODER &=~  GPIO_MODER_MODER8;		
		GPIOA->MODER |= GPIO_MODER_MODER8_0;		// LED
		GPIOA->OTYPER &=~ GPIO_OTYPER_OT_8;			//PUSHPULL
		GPIOA->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR8; 
		GPIOA->MODER &=~  GPIO_MODER_MODER9;		
		GPIOA->MODER |= GPIO_MODER_MODER9_0;		// LED
		GPIOA->OTYPER &=~ GPIO_OTYPER_OT_9;			//PUSHPULL
		GPIOA->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR9; 
		GPIOA->MODER &=~  GPIO_MODER_MODER10;		
		GPIOA->MODER |= GPIO_MODER_MODER10_0;		// LED
		GPIOA->OTYPER &=~ GPIO_OTYPER_OT_10;			//PUSHPULL
		GPIOA->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR10;
		GPIOA->MODER &=~  GPIO_MODER_MODER11;		
		GPIOA->MODER |= GPIO_MODER_MODER11_0;		// LED
		GPIOA->OTYPER &=~ GPIO_OTYPER_OT_11;			//PUSHPULL
		GPIOA->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR11; 
		GPIOA->MODER &=~  GPIO_MODER_MODER12;		
		GPIOA->MODER |= GPIO_MODER_MODER12_0;		// LED
		GPIOA->OTYPER &=~ GPIO_OTYPER_OT_12;			//PUSHPULL
		GPIOA->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR12; 
		GPIOA->MODER &=~  GPIO_MODER_MODER13;		
		GPIOA->MODER |= GPIO_MODER_MODER13_0;		// LED
		GPIOA->OTYPER &=~ GPIO_OTYPER_OT_13;			//PUSHPULL
		GPIOA->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR13; 
		GPIOA->MODER &=~  GPIO_MODER_MODER14;		
		GPIOA->MODER |= GPIO_MODER_MODER14_0;		// LED
		GPIOA->OTYPER &=~ GPIO_OTYPER_OT_14;			//PUSHPULL
		GPIOA->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR14; 
		GPIOA->MODER &=~  GPIO_MODER_MODER15;		
		GPIOA->MODER |= GPIO_MODER_MODER15_0;		// LED
		GPIOA->OTYPER &=~ GPIO_OTYPER_OT_15;			//PUSHPULL
		GPIOA->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR15; 		
		
		GPIOB->MODER &=~  GPIO_MODER_MODER0;		
		GPIOB->MODER |= GPIO_MODER_MODER0_1;		// Iset
		GPIOB->OTYPER &=~ GPIO_OTYPER_OT_0;			//PUSHPULL
		GPIOB->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR0; 
		GPIOB->AFR[0] |= 0x1;		
		

		GPIOB->MODER &=~  GPIO_MODER_MODER1;		
		GPIOB->MODER |= GPIO_MODER_MODER1_0;		// /wr GPIOB->MODER |= GPIO_MODER_MODER1_1;
		GPIOB->OTYPER &=~ GPIO_OTYPER_OT_1;			//PUSHPULL
		GPIOB->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR1;  
		GPIOB->BSRR = GPIO_BSRR_BR_1;
		
		GPIOB->MODER &=~  GPIO_MODER_MODER3;		
		GPIOB->MODER |= GPIO_MODER_MODER3_0;		// d/c
		GPIOB->OTYPER &=~ GPIO_OTYPER_OT_3;			//PUSHPULL
		GPIOB->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR3;  
		GPIOB->BSRR = GPIO_BSRR_BS_3;
		
		GPIOB->MODER &=~  GPIO_MODER_MODER4;		
		GPIOB->MODER |= GPIO_MODER_MODER4_0;		// out en
		GPIOB->OTYPER |= GPIO_OTYPER_OT_4;			
		GPIOB->OSPEEDR |= GPIO_OSPEEDER_OSPEEDR4;  
		GPIOB->BSRR = GPIO_BSRR_BR_4;
		
		GPIOB->MODER &=~  GPIO_MODER_MODER5; 		//Z state
		
		GPIOB->MODER &=~  GPIO_MODER_MODER6;		//I2C_SCL
		GPIOB->OTYPER |= GPIO_OTYPER_OT_6;							     
		GPIOB->MODER |= GPIO_MODER_MODER6_0;		
		GPIOB->BSRR = GPIO_BSRR_BS_6;
	
		GPIOB->MODER &=~  GPIO_MODER_MODER7;		//I2C_SDA
		GPIOB->OTYPER |= GPIO_OTYPER_OT_7;							     
		GPIOB->MODER |= GPIO_MODER_MODER7_0;		
		GPIOB->BSRR = GPIO_BSRR_BS_7;
	};
	
void init_adc(void)
{
	RCC->APB2ENR |= RCC_APB2ENR_ADC1EN; 
	RCC->CR2 |= RCC_CR2_HSI14ON; 
	while ((RCC->CR2 & RCC_CR2_HSI14RDY) == 0)   {  };
	if ((ADC1->CR & ADC_CR_ADEN) != 0) 
		{
		ADC1->CR &= (uint32_t)(~ADC_CR_ADEN);  
		}
	ADC1->CR |= ADC_CR_ADCAL; 
	while ((ADC1->CR & ADC_CR_ADCAL) != 0)   {  };  
	__NOP();__NOP();   
	//ADC1->CFGR1 |= ADC_CFGR1_AUTOFF;
	ADC1->CFGR1 |= ADC_CFGR1_CONT;
	ADC1->CHSELR = ADC_CHSELR_CHSEL0 |ADC_CHSELR_CHSEL1;
	ADC1->SMPR = 0x07; 

	RCC->AHBENR |= RCC_AHBENR_DMA1EN; 
	ADC1->CFGR1 |= ADC_CFGR1_DMAEN | ADC_CFGR1_DMACFG;

	ADC1->CR |= ADC_CR_ADEN; 
	
	DMA1_Channel1->CPAR = (uint32_t) (&(ADC1->DR)); 
	DMA1_Channel1->CMAR = (uint32_t)(ADC_buff); 
	DMA1_Channel1->CNDTR = 2; 
	DMA1_Channel1->CCR |= DMA_CCR_MINC | DMA_CCR_MSIZE_0 | DMA_CCR_PSIZE_0 | DMA_CCR_TEIE | DMA_CCR_CIRC; 
	DMA1_Channel1->CCR |= DMA_CCR_EN; 

	ADC1->CR |= ADC_CR_ADSTART; // start the ADC conversions 
	};

void init_tim3(void)
{
		RCC->APB1ENR |= RCC_APB1ENR_TIM3EN;
		TIM3->PSC = 0;
		TIM3->ARR = Udd;
    TIM3->CCMR1 |= (TIM_CCMR1_OC2M_2|TIM_CCMR1_OC2M_1);
    TIM3->CCMR2 |= (TIM_CCMR2_OC3M_2|TIM_CCMR2_OC3M_1);
    TIM3->CCR2 = 1 ;   // Uset
		TIM3->CCR3 = 1 ;   // Iset
    TIM3->CCER |= TIM_CCER_CC2E |  TIM_CCER_CC3E;
    TIM3->CR1 = TIM_CR1_CEN;
	};

void lcd_send_cmd(uint16_t cmd)
{
    LCD_DC_L;
		LCD_WR_L;
		//GPIOA->BSRR = 0xff000000 | (cmd<<8);
		LCD = cmd;
    LCD_WR_H;
		LCD_DC_H;
};

void lcd_send_data(uint16_t data)
{
		LCD_WR_L;
    LCD = data;
		//GPIOA->BSRR = 0xff000000 | (data<<8);
    LCD_WR_H;
};

void init_lcd(void)
{
	delay_us(100000);
	LCD_DC_H;
  lcd_send_cmd(0x01);        // SW Reset
	delay_us(1000);
  lcd_send_cmd(0x11);
  lcd_send_cmd(0x29);
	delay_us(1000);
	lcd_send_cmd(0x36);
	lcd_send_data(0x00);//c0 2C a0
	lcd_send_cmd(0x3A);
	lcd_send_data(0x05);
};
void fill_screen(uint16_t color)
{
   unsigned int i;
   lcd_send_cmd(0x2A);
   lcd_send_data(0x00);
   lcd_send_data(0x00);
   lcd_send_data(0x00);
   lcd_send_data(175);	
   lcd_send_cmd(0x2B);
   lcd_send_data(0x00);
   lcd_send_data(0x00);
   lcd_send_data(0x00);
   lcd_send_data(207);
   lcd_send_cmd(0x2C);
    for (i=0;i<36608;i++)
    {
      lcd_send_data((color>>8));
      lcd_send_data(color & 0x00ff);
    };
};

void bar(uint8_t y,uint8_t data,uint16_t color)
{
	unsigned int i;
	if (data > 206) data = 206;
   lcd_send_cmd(0x2A);
   lcd_send_data(0x00);
   lcd_send_data(y);
   lcd_send_data(0x00);
   lcd_send_data(y+15);
   lcd_send_cmd(0x2B);
   lcd_send_data(0x00);
   lcd_send_data(207-data);
   lcd_send_data(0x00);
   lcd_send_data(207);
   lcd_send_cmd(0x2C);
    for (i=0;i<(16*data);i++)
    {
      lcd_send_data((color>>8));
      lcd_send_data(color & 0x00ff);
    };	
   lcd_send_cmd(0x2B);
   lcd_send_data(0x00);
   lcd_send_data(0);
   lcd_send_data(0x00);
   lcd_send_data(206-data);	
   lcd_send_cmd(0x2C);
    for (i=0;i<(16*(206-data));i++)
    {
      lcd_send_data(0);
      lcd_send_data(0);
    };		
};

void put_char16(int y,int x, char c,int color,int bgcolor,int mag)
{
	char h,p,m1,m2;
	int mask,ch;
	y = 207-y;
	  lcd_send_cmd(0x2a);
    lcd_send_data(0);        
    lcd_send_data(x);
    lcd_send_data(0);        
    lcd_send_data(x+(mag-1)+15*mag);
    lcd_send_cmd(0x2b);
		lcd_send_data(0);
		lcd_send_data(y+1-11*mag);
    lcd_send_data(0x00);
    lcd_send_data(y);
		lcd_send_cmd(0x2C);
		for (h=11; h>0; h--) 
			{
				ch=ascii_16[c-0x20][h-1];   
				for (m1=mag;m1>0;m1--)
				{
					mask=0x8000;
				for (p=16; p>0; p--) 
          {
						for(m2=mag;m2>0;m2--)
						{
           if (ch&mask)
            {
            lcd_send_data((uint8_t)(color>>8));
            lcd_send_data((uint8_t)color);
            }
           else
            {
            lcd_send_data((uint8_t)(bgcolor>>8));
            lcd_send_data((uint8_t)bgcolor);
            };
						};
						mask=mask>>1;
					};
				 };  
			};		
};

void put_string16(int x,int y, char *str,int color,int bgcolor,int mag)
{
unsigned char  j=0;
  while (j<strlen(str))
   {
   put_char16(x+j*11*mag,y,str[j],color,bgcolor,mag);
   j++;
   }
} ;

void ltoa(long int n,char *str)
 {
 unsigned long i;
 unsigned char j,p;
 i=1000000000L;
 p=0;
 if (n<0)
 {
 n=-n;
 *str++='-';
 };
 do
 {
 j=(unsigned char) (n/i);
 if (j || p || (i==1))
 {
 *str++=j+'0';
 p=1;
 }
 n%=i;
 i/=10L;
 }
 while (i!=0);
 *str=0;
 };
 
void reverse(char s[])
 {
 int i, j;
 char c;
 for (i = 0, j = strlen(s)-1; i<j; i++, j--) {
 c = s[i];
 s[i] = s[j];
 s[j] = c;
 }
 };
 
void itoa(int n, char s[])
 {
 int i, sign;
 if ((sign = n) < 0) //?????????? ???? 
 n = -n; // ?????? n ????????????? ?????? 
 i = 0;
 do { //?????????? ????? ? ???????? ??????? 
 s[i++] = n % 10 + '0'; //????? ????????? ????? 
 } while ((n /= 10) > 0); // ??????? 
 if (sign < 0)
 s[i++] = '-';
 s[i] = '\0';
 reverse(s);
 };

void init_tim16(void)
	{
	RCC->APB2ENR |= RCC_APB2ENR_TIM16EN;           
	TIM16->CR1 |= TIM_CR1_ARPE;
	TIM16->CR2 |= TIM_CR2_OIS1;
	TIM16->CCMR1 |= TIM_CCMR1_OC1M_2 | TIM_CCMR1_OC1M_1;	
	TIM16->CCMR1 |= TIM_CCMR1_OC1FE;
	TIM16->CCER |= TIM_CCER_CC1E;
	TIM16->PSC = 64;			
	TIM16->ARR = 1515-1;					//2273 ->440Hz	1515->660Hz
	TIM16->CCR1 = TIM16->ARR>>1;	//duty
	TIM16->BDTR |= TIM_BDTR_MOE;	
	};

void beep(uint16_t time)
	{
		GPIOA->MODER |= GPIO_MODER_MODER6_1;TIM16->CR1 |= TIM_CR1_CEN;
		delay_us(time * 1000);
		TIM16->CR1 &=~ TIM_CR1_CEN;GPIOA->MODER &=~  GPIO_MODER_MODER6;
		GPIOA->BSRR = GPIO_BSRR_BR_6;
	};

void save_seTings (void)
	{
		reserv[0] = Uset >> 8;
		reserv[1] = Uset & 0xff;
		reserv[2] = Iset >> 8;
		reserv[3] = Iset & 0xff;
		reserv[4] = S >> 24;
		reserv[5] = (S >> 16) & 0xff;
		reserv[6] = (S >> 8) & 0xff;
		reserv[7] = S & 0xff;
		reserv[8] = Gmode;
		save_const(32);
	};
void load_seTings(void)
	{
		load_const(32);
		Uset = (((uint16_t)reserv[0]) << 8) | reserv[1];
		Iset = (((uint16_t)reserv[2]) << 8) | reserv[3];
		S = (((uint32_t)reserv[4]) << 24) | (((uint32_t)reserv[5]) << 16) | (((uint32_t)reserv[6]) << 8) | reserv[7];
		Gmode = reserv[8];
	};

void SysTick_Handler(void)
	{

		adc_sum[0] = adc_sum[0] - adc_data[0][ADC_ptr];
		adc_sum[1] = adc_sum[1] - adc_data[1][ADC_ptr];
		adc_data[0][ADC_ptr] = ADC_buff[0];
		adc_data[1][ADC_ptr] = ADC_buff[1];
		adc_sum[0] = adc_sum[0] + adc_data[0][ADC_ptr];
		adc_sum[1] = adc_sum[1] + adc_data[1][ADC_ptr];
		if (++ADC_ptr > 31) ADC_ptr = 0;
		adc_res[0] = adc_sum[0] >> 5;
		adc_res[1] = adc_sum[1] >> 5;
		
		In = (adc_res[1] - I0);
		if (In < 0) In = 0; else  In = (In*Ki)>>8;
		Un = ((adc_res[0]*Ku)>>8)-((In*Rsht)>>13);	//59
		if (++dd_sec > 9) 
		{
			dd_sec = 0,Dsec_fl = 1;
			S = S + In;
			if (S > 3599964000) S = 0;
		};
	}; 
	
int main(void)
{	
		RCC->CR |= RCC_CR_HSEON;
		while (!(RCC->CR & RCC_CR_HSERDY)) {};
		FLASH->ACR |= FLASH_ACR_PRFTBE;
		FLASH->ACR |= FLASH_ACR_LATENCY;	
		RCC->CFGR |= RCC_CFGR_PLLSRC_HSE_PREDIV;
		RCC->CFGR |= RCC_CFGR_PLLXTPRE;
		RCC->CFGR2 = (4-1);								// 	26/4
		RCC->CFGR |= (10-2)<<18; 					/// Xtall/4*10=65
		RCC->CR |= RCC_CR_PLLON;
		while (!(RCC->CR & RCC_CR_PLLRDY)) {}; 
		RCC->CFGR |= RCC_CFGR_SW_PLL;
		RCC->CFGR |= RCC_CFGR_HPRE_DIV1;    
		RCC->CFGR |= RCC_CFGR_PPRE_DIV1;
		RCC->APB2ENR |= RCC_APB2ENR_SYSCFGEN;
		init_gpio();	
		init_tim3();
		init_adc();
		init_tim16();
		init_lcd();fill_screen(0);
		load_seTings();
		SysTick_Config(650000); // 0.01 sek	

		put_char16(19,16,'.',Ucol,black,1);put_char16(49,16,'V',Ucol,black,1);
		put_char16(121,0,'.',Ucol,black,3);
		put_char16(8,64,'.',Icol,black,1);put_char16(49,64,'A',Icol,black,1);
		put_char16(88,48,'.',Icol,black,3);	
		put_string16(0,112,"VOLUME",Scol,black,1);put_string16(174,128,"mAh",Scol,black,1);
		
	while(1)
	{
		Key_pow = (GPIOA->IDR & 0x04) >> 2;
		if(Key_pow == 0) save_seTings(),beep(100),delay_us(10000000);
		Guard = (GPIOB->IDR & 0x20) >> 5;
		if (Guard) 
			{
				OutEN = 0,GPIOB->BSRR = GPIO_BSRR_BR_4,TIM3->CCR2 = 0;
				beep(1000);				
			};
		if ((Gmode == TRIG) && (In >= Iset)) GPIOB->BSRR = GPIO_BSRR_BR_4,beep(2000),GPIOB->BSRR = GPIO_BSRR_BS_4;
		if (Dsec_fl) 
		{
			Dsec_fl = 0;
			Key = ((~GPIOA->IDR)>>3) & 0x07;
			if (Key)
			{
			if (Key == 3)  {if (++set_mode > 3) set_mode = 0;beep(200);};
			if (set_mode == USET) {
			switch (Key)
			{
				case 1: {if (Uset < 1901) Uset = Uset + 100;};break;
				case 2: {if (Uset > 100) Uset = Uset - 100;if (Uset < U0set) Uset = U0set;};break;
				case 6: {if (Uset > U0set) Uset = Uset - 1;};break;
				case 5: {if (Uset < 2000) Uset = Uset + 1;};break;
				case 4: {if (OutEN) OutEN = 0; else OutEN = 1;};break;
			};
			} else
			if (set_mode == ISET) {
			switch (Key)
			{
				case 1: {if (Iset < 1901) Iset = Iset + 100;};break;
				case 2: {if (Iset > 100) Iset = Iset - 100;};break;
				case 6: {if (Iset > 1) Iset = Iset - 1;};break;
				case 5: {if (Iset < 1999) Iset = Iset + 1;};break;
				case 4: {if (OutEN) OutEN = 0; else OutEN = 1;};break;
			};
			} else
			if (set_mode == SSET) {
			switch (Key)
			{
				case 1:
				case 2: {S = 0;put_string16(64,112,"     ",Scol,black,2);};break;
				case 4: {if (OutEN) OutEN = 0; else OutEN = 1;};break;
			};
			} else {
			switch (Key)
			{
				case 5: {Gmode = TRIG;};break;
				case 6: {Gmode = CONST;};break;
				case 4: {if (OutEN) OutEN = 0; else OutEN = 1;};break;
			};
			};		
			};
			TIM3->CCR3 = ((Iset*KIset)>>8) + I0set; // Iset
			if (OutEN) 
				{
					TIM3->CCR2 = Uset - U0set - ((KUset*Uset) >> 7) + ((In*Rsht)>>13);
					put_string16(119,144," ON ",black,red,2);
					GPIOB->BSRR = GPIO_BSRR_BS_4;
				}
				else GPIOB->BSRR = GPIO_BSRR_BR_4,TIM3->CCR2 = 0,put_string16(119,144,"    ",red,black,2);
			if (Key == 4) beep(500); else if (Key != 0) beep(20);
				
			if (set_mode == USET) put_string16(6,0,"U SET",black,Ucol,1); else put_string16(6,0,"U SET",Ucol,black,1);
			if (set_mode == ISET) put_string16(6,48,"I SET",black,Icol,1); else put_string16(6,48,"I SET",Icol,black,1);
			if (set_mode == SSET) put_string16(0,128,"CLEAR",black,Scol,1); else put_string16(0,128,"     ",Scol,black,1);
			if (set_mode == GSET) 
			{
				if (Gmode == TRIG) put_string16(0,144,"TRIGG",black,Gcol,2); else put_string16(0,144,"CONST",black,Gcol,2);	
			} else {
				if (Gmode == TRIG) put_string16(0,144,"TRIGG",Gcol,black,2); else put_string16(0,144,"CONST",Gcol,black,2);
			};
			
			itoa(Uset,txt);stln = strlen(txt);
			if (stln < 2) put_char16(0,16,' ',Ucol,black,1),put_char16(11,16,'0',Ucol,black,1),put_char16(27,16,'0',Ucol,black,1),put_char16(38,16,txt[0],Ucol,black,1); else
				if (stln < 3) put_char16(0,16,' ',Ucol,black,1),put_char16(11,16,'0',Ucol,black,1),put_char16(27,16,txt[0],Ucol,black,1),put_char16(38,16,txt[1],Ucol,black,1); else
					if (stln < 4) put_char16(0,16,' ',Ucol,black,1),put_char16(11,16,txt[0],Ucol,black,1),put_char16(27,16,txt[1],Ucol,black,1),put_char16(38,16,txt[2],Ucol,black,1); else
						put_char16(0,16,txt[0],Ucol,black,1),put_char16(11,16,txt[1],Ucol,black,1),put_char16(27,16,txt[2],Ucol,black,1),put_char16(38,16,txt[3],Ucol,black,1);
			itoa(Un,txt);stln = strlen(txt);
			if (stln < 2) put_char16(66,0,' ',Ucol,black,3),put_char16(99,0,'0',Ucol,black,3),put_char16(142,0,'0',Ucol,black,3),put_char16(175,0,txt[0],Ucol,black,3); else
				if (stln < 3) put_char16(66,0,' ',Ucol,black,3),put_char16(99,0,'0',Ucol,black,3),put_char16(142,0,txt[0],Ucol,black,3),put_char16(175,0,txt[1],Ucol,black,3); else
					if (stln < 4) put_char16(66,0,' ',Ucol,black,3),put_char16(99,0,txt[0],Ucol,black,3),put_char16(142,0,txt[1],Ucol,black,3),put_char16(175,0,txt[2],Ucol,black,3); else
						put_char16(66,0,txt[0],Ucol,black,3),put_char16(99,0,txt[1],Ucol,black,3),put_char16(142,0,txt[2],Ucol,black,3),put_char16(175,0,txt[3],Ucol,black,3);
			itoa(Iset,txt);stln = strlen(txt);
			if (stln < 2) put_char16(0,64,'0',Icol,black,1),put_char16(16,64,'0',Icol,black,1),put_char16(27,64,'0',Icol,black,1),put_char16(38,64,txt[0],Icol,black,1); else
				if (stln < 3) put_char16(0,64,'0',Icol,black,1),put_char16(16,64,'0',Icol,black,1),put_char16(27,64,txt[0],Icol,black,1),put_char16(38,64,txt[1],Icol,black,1); else
					if (stln < 4) put_char16(0,64,'0',Icol,black,1),put_char16(16,64,txt[0],Icol,black,1),put_char16(27,64,txt[1],Icol,black,1),put_char16(38,64,txt[2],Icol,black,1); else
						put_char16(0,64,txt[0],Icol,black,1),put_char16(16,64,txt[1],Icol,black,1),put_char16(27,64,txt[2],Icol,black,1),put_char16(38,64,txt[3],Icol,black,1);
			itoa(In,txt);stln = strlen(txt);
			if (stln < 2) put_char16(66,48,'0',Icol,black,3),put_char16(109,48,'0',Icol,black,3),put_char16(142,48,'0',Icol,black,3),put_char16(175,48,txt[0],Icol,black,3); else
				if (stln < 3) put_char16(66,48,'0',Icol,black,3),put_char16(109,48,'0',Icol,black,3),put_char16(142,48,txt[0],Icol,black,3),put_char16(175,48,txt[1],Icol,black,3); else
					if (stln < 4) put_char16(66,48,'0',Icol,black,3),put_char16(109,48,txt[0],Icol,black,3),put_char16(142,48,txt[1],Icol,black,3),put_char16(175,48,txt[2],Icol,black,3); else
						 put_char16(66,48,txt[0],Icol,black,3),put_char16(109,48,txt[1],Icol,black,3),put_char16(142,48,txt[2],Icol,black,3),put_char16(175,48,txt[3],Icol,black,3);
			itoa(S/36000,txt);stln = strlen(txt);
			if (stln == 5) put_string16(64,112,txt,Scol,black,2);else
				if (stln == 4) put_string16(86,112,txt,Scol,black,2);else
					if (stln == 3) put_string16(108,112,txt,Scol,black,2);else
						if (stln == 2) put_string16(130,112,txt,Scol,black,2);else
							put_string16(152,112,txt,Scol,black,2);
					
		p = 0;temp = 2000;		//logarifm bar
		if (In <= 2000) {while (temp > In) {p++;temp = (temp * 986)>>10;};};	//986 = 1024/( 200   2000)   
		if (p > 99) p = 99;p = 200 - (p<<1);		//  = 300
		bar(97,p,red);
		
		};			
	};		
}
